#!/usr/bin/python

"""
Gravis Gallery Generator (ver i don't have the head to maintain version numbers)

This creates the most basic static gallery imaginable. The only frills are an optional js library which you can just ignore if you want. There's no templating system or anything just edit the HTML.

	IF YOU EXECUTE THIS WITHOUT BEING IN THE GALLERY DIRECTORY YOU WILL CLOBBER YOUR WEBSITE.
	DO NOT EXECUTE THIS FROM OUTSIDE THE DIRECTORY.
	THAT MEANS DON'T DO THIS: ./gallery/gallery.py

Instructions:
	This script needs to be in a directory with all the files it came with
	Make a subdirectory for every album you want
	Put all the images you want to display in each subdirectory
	Now run the script from within the directory: ./gallery.py
	128x128 thumbnails will be generated for everything. Change that size below if you want.
	Thumbnails will be stored in <album folder>/thumbs. That folder will be created automatically.
	An index.html will be created with the gallery interface.
		ANY EXISTING INDEX PAGE WILL BE OVERWRITTEN.
	Finally, an index.html will be created next to where this script lives with links to each album
	That's it!

If you want to use this it's probably pretty safe and simple.
I run it on Dreamhost so it will probably run anywhere. Just needs PIL.
"""

# Change this to alter the thumbnail size:
size = 128, 128
# Change this to alter the number of images per row
row_max = 6
# Set this to True if you don't want to see a message for thumbnails that already exist
thumb_verbose = False

# HTML template for the album page (the one that actually shows the pictures)
h_album_template = """
<head>
	<link rel="stylesheet" href="../gallery.css">
	<link rel="stylesheet" href="../baguetteBox.min.css">
	<script src="../baguetteBox.min.js" async></script>
	<title>{title} - Bramble Gallery</title>
</head>
<body>
	<div class="wrapper">
	<h1>Bramble Gallery</h1>
	<h2>{title}</h2>
	
	<label for="baguette">Fancy javascript</label><input type="checkbox" id="baguette">
	<br><small>&nearr;Good for phones! You can swipe!&nwarr;</small>
	
	{imagetable}
	
	<a href="../">&laquo; Back to Gallery</a>
	</div>
	<script>
		{js}
	</script>
</body>
	"""

# HTML template for the index page (the one that lists each album)
h_index_template = """
<head>
	<link rel="stylesheet" href="gallery.css">
	<title>Bramble Gallery</title>
</head>
<body>
	<div class="wrapper">
	<h1>Bramble Gallery</h1>
	<h2>Index</h2>
	
	{albumlinks}
	
	<a href="https://gekk.info/code/gallery.zip">Download gallery script</a><br>
	<a href="/">&laquo; Back to Home</a>
	</div>
</body>
	"""

# HTML template for each link on the index page
h_index_link_template = """<a href="{dir}">{dir}</a><br>\n"""
	
# HTML template for each picture in the gallery table
h_gallery_cell_template = """<td>
		<a href="{file}"><img src="thumbs/{file}"></a>
		</td>"""

# Gallery javascript, a separate string because escaping {} in python format is irritating
js = """document.getElementById("baguette").onclick = function(event) {
			if (event.target.checked == true) {
				baguetteBox.run("table");
			} else {
				baguetteBox.destroy();
			}
	}"""
		
# ==============================================================================
import os
from os.path import join, exists
from PIL import Image, ExifTags

# this will be the list of album links in the root index
h_index = ""

# walk each folder in the current path
for dir in os.listdir('./'):
	path = dir + "/"
	
	# ignore non-directories
	if not os.path.isdir(path):
		continue
	
	print "Processing " + dir
	
	# thumbnails will always be stored under the parent folder
	thumbs = path + 'thumbs/'
	
	# make the thumbnail folder if need be
	if not os.path.exists(thumbs):
		os.makedirs(thumbs)
	
	
	# list of files to go in the image table
	l_files = []
	
	# make thumbnails for every file
	for file in os.listdir(path):
		# ignore index.html and thumbs of course
		if file in ["index.html", "thumbs"]:
			continue
		fpath = path + file
		fthumb = thumbs + file
		
		# This flag is true unless the file fails to be processed
		thumb_valid = True
		
		# only make a thumb if one doesn't already exist
		if not os.path.exists(fthumb):
			# If anything ends up in the folder that PIL can't understand, this will error.
			# We print an error and continue, nbd
			try:
				image = Image.open(fpath)
				
				# This code would rotate images that need it and overwrite the original file
				# That would fix orientation for the image and the thumbnail, but results in a loss
				# of quality.
				# You could uncomment this, but I have not tested it since it turns out my test files
				# are from a camera that predates orientation sensors.
				# Enable at your own risk.
				"""
				# First we'll check if the image needs rotation
				for orientation in ExifTags.TAGS.keys():
					if ExifTags.TAGS[orientation]=='Orientation':
						break
				exif=dict(image._getexif().items())

				print (exif)
				print (orientation)
				if exif[orientation] in [3,6,8]:
					print ("Rotating " + file)
					if exif[orientation] == 3:
						image=image.rotate(180, expand=True)
					elif exif[orientation] == 6:
						image=image.rotate(270, expand=True)
					elif exif[orientation] == 8:
						image=image.rotate(90, expand=True)
					image.save(fpath)"""
				
				print "Creating thumbnail for " + file
				image.thumbnail(size)
				image.save(fthumb, "JPEG")
			except NameError as detail:
				print ("Error: " + str(detail))
				# Invalidate this thumb so it doesn't get a broken image in the gallery
				thumb_valid = False
		else:
			if thumb_verbose is True:
				print "Thumbnail already exists for " + file
		
		# This will be false if the file failed to read
		if thumb_valid is True:
			l_files.append(file)
	
	# If there were no images in this folder, skip it
	if len(l_files) == 0:
		print ("No valid images in " + dir + " - skipping")
		continue
	
	# Add a link to the album index
	h_index = h_index + h_index_link_template.format(dir=dir)	
	
	# create the image table in rows of five
	# This leaves an empty row at the bottom. I could add code to fix this but it's not worth it to me.
	x = row_max
	
	# this will be the HTML representation of the image table
	h_table = "<table><tr>"
	for file in l_files:
		if x >= row_max:
			# end row and start a new one
			h_table = h_table + "</tr>\n<tr>"
			x = 0
		h_table = h_table + h_gallery_cell_template.format(file=file)
		x = x + 1
	# add the end tags to the table
	h_table = h_table + "</tr></table>"
		
	# now let's create the gallery page
	html = h_album_template.format(title=dir, imagetable=h_table, js=js)
	# and write it to disk
	h_out = open(path + "/index.html", "w")
	h_out.write(html)
	h_out.close()

# now let's write the master index	
html = h_index_template.format(title=dir, albumlinks=h_index)
# and write it to disk
h_out = open("./index.html", "w")
h_out.write(html)
h_out.close()

print ("All done!")